/*
    
	DAC Demo, ATTiny3216, Ausgabe auf PA6 - Sgezahl

*/

/* 3.33 MHz (needed for delay) */
#define F_CPU                         (3333333UL)

/* DAC value */
#define DAC_EXAMPLE_VALUE             (0x20)

/* VREF Startup time */
#define VREF_STARTUP_MICROS           (25)


#include <avr/io.h>
#include <util/delay.h>

void VREF_init(void);
void DAC0_init(void);
void DAC0_setVal(uint8_t val);

void VREF_init(void)
{
    /* Voltage reference at 4.34V */
    VREF.CTRLA |= VREF_DAC0REFSEL_4V34_gc;
	
    /* DAC0/AC0 reference enable: enabled */
    VREF.CTRLB |= VREF_DAC0REFEN_bm;
	
    /* Wait VREF start-up time */
    _delay_us(VREF_STARTUP_MICROS);
}

void DAC0_init(void)
{
    /* Disable digital input buffer */
    PORTA.PIN6CTRL &= ~PORT_ISC_gm;
    PORTA.PIN6CTRL |= PORT_ISC_INPUT_DISABLE_gc;
	
    /* Disable pull-up resistor */
    PORTA.PIN6CTRL &= ~PORT_PULLUPEN_bm;
    
    /* Enable DAC, Output Buffer, Run in Standby */
    DAC0.CTRLA = DAC_ENABLE_bm | DAC_OUTEN_bm | DAC_RUNSTDBY_bm;
}

void DAC0_setVal(uint8_t val)
{
    DAC0.DATA = val;
}

int main(void)
{
    VREF_init();
    DAC0_init();
    DAC0_setVal(DAC_EXAMPLE_VALUE);
    
    while (1) 
    {
		;
		
		// Sgezahn ( Rampe auf und ab)
		// Rampe auf
        for(int i=0;i<256;i++) 
		{
			DAC0_setVal(i);
			_delay_us(10);
		}
		
		//Rampe ab
		for(int i=255;i>0;i--)
		{
			DAC0_setVal(i);
			_delay_us(10);
		}

		
    }
}
