/*
    \file   main.c

    Using DAC as negative input for AC

    (c) 2018 Microchip Technology Inc. and its subsidiaries.

    
*/

/* 3.33 MHz (needed for delay) */
#define F_CPU                         (3333333UL)
/* 1.4V output @ VREF = 1.5V */
#define DAC_DATA_1V4                  (239)
/* VREF Startup time */
#define VREF_STARTUP_MICROS           (25)

#include <avr/io.h>
#include <util/delay.h>

void VREF_init(void);
void DAC0_setVal(uint8_t val);
void DAC0_init(void);
void AC0_init(void);

void VREF_init(void)
{
    /* Voltage reference at 1.5V */
    VREF.CTRLA |= VREF_DAC0REFSEL_1V5_gc;
    /* DAC0/AC0 reference enable: enabled */
    VREF.CTRLB |= VREF_DAC0REFEN_bm;
    /* Wait VREF start-up time */
    _delay_us(VREF_STARTUP_MICROS);
}

void DAC0_setVal(uint8_t val)
{
    DAC0.DATA = val;
}

void DAC0_init(void)
{
    /* Disable digital input buffer */
    PORTA.PIN6CTRL &= ~PORT_ISC_gm;
    PORTA.PIN6CTRL |= PORT_ISC_INPUT_DISABLE_gc;
    /* Disable pull-up resistor */
    PORTA.PIN6CTRL &= ~PORT_PULLUPEN_bm;
    
    /* Enable DAC, Output Buffer, Run in Standby */
    DAC0.CTRLA = DAC_ENABLE_bm | DAC_OUTEN_bm | DAC_RUNSTDBY_bm;
}

void AC0_init(void)
{
    /* Negative input from DAC0 */
    /* Positive input from pin PA7 */
    AC0.MUXCTRLA = AC_MUXNEG_DAC_gc
                 | AC_MUXPOS_PIN0_gc;
    
    /* Enable, Output on PA5, Low Power mode */
    AC0.CTRLA = AC_ENABLE_bm
              | AC_OUTEN_bm
              | AC_RUNSTDBY_bm
              | AC_LPMODE_bm;
}

int main(void)
{
    /* Voltage divider -> VDD/2 input on PA7 */
    /* AC output on LED on PA5 */
    /* LED turns OFF when battery is below 2.8V (PA7 below 1.4V) */
    
    VREF_init();
    DAC0_init();
    AC0_init();

    /* 1.4V output @ VREF = 1.5V */
    DAC0_setVal(DAC_DATA_1V4);
    
    while (1)
    {
        ;
    }
}
