/*
 * ATTiny3216_RTC_Test.cpp
 *
 * Created: 05.08.2020 22:37:35
 * Author : Micha
 *
 * Spielerei mit dem RTC im ATtiny3216:
 * Jede Minute wird die Variable "minute" um eins erhht und via USART angezeigt.
 * luft!
 */ 

/* Atmel Studio 7 - nderung bei den Parametern des Linkers: 
  Zur Benutzung von sprintf() (u.a.) und float / double-Variablen - nderungen beim Linker:
- Men Projekt, Properties zu diesem Projekt aufrufen
- unter Toolchain / Linker / General die Option  "Use vprintf-library" anklicken
- unter Toolchain / Linker / Miscellaneous in das Feld eintragen: -lprintf_flt
- nun kann man sprintf() und printf() auch benutzen mit float und double
- Quelle: https://startingelectronics.org/articles/atmel-AVR-8-bit/print-float-atmel-studio-7/
*/

#ifndef F_CPU
#define F_CPU 3333333UL								// 3,3 MHz
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

volatile uint16_t minute=0;
char s[80];


void CLKCTRL_init(void)
{
	#if (F_CPU == 1000000UL)                     /* 1MHz, 16/16, set fuse for 16 MHz */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_16X_gc | CLKCTRL_PEN_bm);
	#endif
	
	#if (F_CPU == 20000000UL)                  /* 20MHz, no division */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);
	#endif
	
	#if (F_CPU == 3333333UL)					/* 3.3 MHz bei Fuse = 20 MHz`*/
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSC20M_gc);
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */
	#endif

}

// schaltet die MAIN_CLK auf internen Oszillator 32,768 kHz
void Main_Clock_32_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);	// Prescaler aus
}

// schaltet die MAIN_CLK auf internen Oszillator 32 kHz mit Prescaler 32 = 32 / 32 = 1 kHz
void Main_Clock_1_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_32X_gc | CLKCTRL_PEN_bm);

}

// schaltet die MAIN_CLK auf internen Oszillator 32 kHz mit Prescaler 32 = 32 / 64 = 0.5 kHz
void Main_Clock_0_5_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_64X_gc | CLKCTRL_PEN_bm);

}

// schaltet die Main_CLK auf intern 16 bzw. 20 MHz, je nach Einstellung der FUSE
void Main_Clock_20_MHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSC20M_gc);
}

// ===========================================================================================
// USART0:
// ===========================================================================================

//#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <avr/interrupt.h>

#define USART0_BAUD_RATE(BAUD_RATE) ((float)(F_CPU * 64 / (16 * (float)BAUD_RATE)) + 0.5)

// 9600 Baud
void USART0_init(void)
{

	PORTB.DIRCLR = PIN3_bm;		// PORTB.3 = INPUT = RxD
	PORTB.DIRSET = PIN2_bm;		// PORTB.2 = OUTPUT = TxD

	USART0.BAUD = (uint16_t)USART0_BAUD_RATE(9600);				// bei F_CPU = 1 Mhz oder 3,3 MHz
	//USART0.BAUD = (uint16_t)USART0_BAUD_RATE(115200);
	
	USART0.CTRLB |= USART_TXEN_bm;          /* Transmitter Enable bit mask. */

}

static void USART0_sendChar(char c)
{
	while (!(USART0.STATUS & USART_DREIF_bm))
	{
		;                                   /* Wait for USART ready for receiving next char */
	}
	USART0.TXDATAL = c;
}

//Anwendung: USART0_sendString( (char *) "-Text-\r\n");
void USART0_sendString(char *str)
{
	uint8_t i;
	for(i = 0; i < strlen(str); i++)
	{
		USART0_sendChar(str[i]);
	}
}

// Debugging: eine Variable anzeigen
void debug_int(uint16_t wert)
{
	uint8_t i= 0;
	char s[10] = "";
	// char * itoa(int val,char * s,int radix)
	itoa(wert,s,10);
	for(uint8_t i=0;i<9;i++)			// CR an die Ziffern anhngen
	{
		if (s[i] == 0)
		{
			s[i]= 13;
			s[i+1] = 10;
			break;
		}
	}
	do
	{
		USART0_sendChar(s[i]);	// wait until TX can send
	} while (s[++i] != 0);
}

//void usart0_status()
//{
//sprintf(s,"->->Status = %d Intflags: %d \r\n", status, intflags);
//USART0_sendString( (char *) s);
//}


// ----------------------------------------------------------------------------------
// RTC ------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------

void RTC_init(void)
{
	while (RTC.STATUS > 0);					// Wait for all register to be synchronized
	RTC.CLKSEL |= RTC_CLKSEL_INT1K_gc;		// 32KHz divided by 32, i.e run at 1.024kHz
	RTC.PER = 0xF000;						// is Overflow-Value 
	//RTC.PERL = 0x00;						// 0xF000 triggers every 61440 cycles = 61440 / 1.024kHz = 60 seconds
	//RTC.PERH = 0xF0;						// 
	RTC.INTCTRL = RTC_OVF_bm;				// Overflow interrupt
	RTC.CTRLA |= RTC_PRESCALER_DIV1_gc		// NO Prescaler
			| RTC_RTCEN_bm			       	// Enable RTC
			| RTC_RUNSTDBY_bm;   			// Run in standby also
}

ISR(RTC_CNT_vect)
{
	minute++;
	RTC.INTFLAGS = RTC_OVF_bm;
}

// ----------------------------------------------------------------------------------
// Main -----------------------------------------------------------------------------
// ----------------------------------------------------------------------------------

int main(void)
{
	CLKCTRL_init();
	USART0_init();
	RTC_init();
	sei();
	
    /* Replace with your application code */
    while (1) 
    {
		sprintf(s,"Minute: %u \r\n", minute);
		USART0_sendString(s);
		_delay_ms(5000);
    }
}

