#ifndef _adc0_
#define _adc0_

// AtTiny3216/817 etc.
// ===========================================================================================
// ADC: ADC-Input ist PA6
// ===========================================================================================


// Änderungen der vordefinierten Parameter in Visual Studio /Atmel Studio 7:
// Mit Maus Wort, z.B. ADC_REFSEL_VDDREF_g markieren, dann rechte Maustaste, dann
// "Goto Implementation" aufrufen (oder ALT-G). Im nun zu sehenden Text sind alternative
// Einstellungen zu sehen.

void ADC0_init(void)
{
	/* Disable digital input buffer */
	PORTA.PIN6CTRL &= ~PORT_ISC_gm;					// Reset Bit 
	PORTA.PIN6CTRL |= PORT_ISC_INPUT_DISABLE_gc;	// Set Bit  
	
	/* Disable pull-up resistor */
	PORTA.PIN6CTRL &= ~PORT_PULLUPEN_bm;			// Reset Bit 

	ADC0.CTRLC = ADC_PRESC_DIV256_gc    /* CLK_PER divided by 256 (zuvor: 4) */
	| ADC_REFSEL_VDDREF_gc;				/* VDD reference */
	
	ADC0.CTRLA = ADC_ENABLE_bm          /* ADC Enable: enabled */
	| ADC_RESSEL_10BIT_gc;				/* 10-bit mode */
	
	/* Select ADC channel 6 */
	ADC0.MUXPOS  = ADC_MUXPOS_AIN6_gc;
}

uint16_t ADC0_read(void)
{
	/* Start ADC conversion */
	ADC0.COMMAND = ADC_STCONV_bm;
	
	/* Wait until ADC conversion done */
	while ( !(ADC0.INTFLAGS & ADC_RESRDY_bm) )
	{
		;
	}
	
	/* Clear the interrupt flag by writing 1: */
	ADC0.INTFLAGS = ADC_RESRDY_bm;
	
	return ADC0.RES;
}

// =======================================================================================
// Anwendung:

// #include <util/delay.h>
// uint16_t adcVal;				// Wert aus ADC-Wandlung

// int main(void)
// {
	
	// ADC0_init();
   
    // while (1) 
    // {
		// adcVal = ADC0_read();
		// // ...
		// _delay_ms(1000);
    // }
// }

#endif





// ====================================================================================