
// MCP79410 RTC schreiben und lesen by mw
// Hardware_ Attiny3216

#ifndef _mcp79410_h
#define _mcp79410_h

// benötigt I2C-Lib für Attiny3216
#include "I2C_TWI.h"

/* globale Variablen */
int minute, stunde, tag, monat, jahr;

// MCP79410 I2C Adresse
#define MCP79410_Write_Addr 0xDE
#define MCP79410_Read_Addr 0xDF

/* Konvertiere BCD <-> Dezimal */
uint8_t dec2bcd(byte val)
{ return ((val/10*16) + (val%10)); }

/* Konvertiere Dezimal <-> BCD */
uint8_t bcd2dec(byte val)
{ return ((val/16*10) + (val%16)); }
	
	
uint8_t MCP79410_ReadRTCByte(uint8_t adr)
{
  unsigned char data;
  TWI_Start(MCP79410_Write_Addr);
  TWI_Write(adr);
  TWI_Start(MCP79410_Read_Addr);
  data = TWI_Read(NACK);
  TWI_Stop();
  return bcd2dec(data);
}
 
void MCP79410_WriteRTCByte(uint8_t adr, uint8_t data)
{
  TWI_Start(MCP79410_Write_Addr);
  TWI_Write(adr);
  TWI_Write(data);
  TWI_Stop();
} 

  
void MCP79410_DateTime_write(uint16_t jahr, uint8_t monat, uint8_t tag, uint8_t stunde, uint8_t minute)
{
	MCP79410_WriteRTCByte(0,0);					//STOP RTC
    MCP79410_WriteRTCByte(7,0b01000011);		// SQWEN = 1, MFP-Pin = 32.768 Hz Output
	MCP79410_WriteRTCByte(1,dec2bcd(minute));    //MINUTE
	MCP79410_WriteRTCByte(2,dec2bcd(stunde));    //HOUR
	MCP79410_WriteRTCByte(3,0x09);				//DAY=1(MONDAY) AND VBAT=1
	MCP79410_WriteRTCByte(4,dec2bcd(tag));		//DAY
	MCP79410_WriteRTCByte(5,dec2bcd(monat));    //MONTH
	MCP79410_WriteRTCByte(6, dec2bcd((uint8_t)jahr-2000));    //YEAR
	MCP79410_WriteRTCByte(0,0x80);				//START RTC, SECOND=00
}

uint8_t MCP79410_Get_Seconds()
{
	  unsigned char data;
	  TWI_Start(MCP79410_Write_Addr);
	  TWI_Write(0);
	  TWI_Start(MCP79410_Read_Addr);
	  data = TWI_Read(NACK);
	  TWI_Stop();
	  data &= 0b01111111;				// oberstes Bit löschen, gehört nicht zu den Sekunden
	  return (bcd2dec(data));
}

uint8_t MCP79410_Get_Minutes()
{
	return MCP79410_ReadRTCByte(0x01);		// Minute auslesen
}

uint8_t MCP79410_Get_Hours()
{
	return MCP79410_ReadRTCByte(0x02);
}

uint8_t MCP79410_Get_Day()
{
	return MCP79410_ReadRTCByte(0x04);
}

uint8_t MCP79410_Get_Month()
{
	return MCP79410_ReadRTCByte(0x05);
}

uint16_t MCP79410_Get_Year()
{
	return (MCP79410_ReadRTCByte(0x06)+2000);
}

#endif