
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>


// Änderungen der vordefinierten Parameter in Visual Studio /Atmel Studio 7:
// Mit Maus Wort, z.B. USART_TXEN_bm markieren, dann rechte Maustaste, dann
// Funktion "Goto Implementation" (oder ALT-G) aufrufen. 
// Im nun zu sehenden Text sind alternative Einstellungen zu sehen.


void CLKCTRL_init(void)
{
	#if (F_CPU == 1000000UL)                     /* 1MHz, 16/16, set fuse for 16 MHz */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_16X_gc | CLKCTRL_PEN_bm);
	#endif 
	
	#if (F_CPU == 20000000UL)                  /* 20MHz, no division */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);
	#endif
	
	#if (F_CPU == 3333333UL)					/* 3.3 MHz bei Fuse = 20 MHz`*/
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */
	#endif
	

}

// ===========================================================================================
// USART0:
// ===========================================================================================
#define USART0_BAUD_RATE(BAUD_RATE) ((float)(F_CPU * 64 / (16 * (float)BAUD_RATE)) + 0.5)

void USART0_init(void)
{

	PORTB.DIRCLR = PIN3_bm;		// PORTB.3 = INPUT = RxD
	PORTB.DIRSET = PIN2_bm,		// PORTB.2 = OUTPUT = TxD

	USART0.BAUD = (uint16_t)USART0_BAUD_RATE(9600);		// bei F_CPU = 1 Mhz
	//USART0.BAUD = (uint16_t)USART0_BAUD_RATE(115200);
	
	USART0.CTRLB |= USART_TXEN_bm;          /* Transmitter Enable bit mask. */

}

static void USART0_sendChar(char c)
{
	while (!(USART0.STATUS & USART_DREIF_bm))
	{
		;                                   /* Wait for USART ready for receiving next char */
	}
	USART0.TXDATAL = c;
}
// Anwendung: z.B. USART0_sendString((char *) "Hallo\r\n");

void USART0_sendString(char *str)
{
	for(size_t i = 0; i < strlen(str); i++)
	{
		USART0_sendChar(str[i]);
	}
}


// Debugging: eine Variable anzeigen 
void debug_int(uint16_t wert)
{
	uint8_t i= 0;
	char s[10] = "";
	// char * itoa(int val,char * s,int radix)
	itoa(wert,s,10);
	for(uint8_t i=0;i<9;i++)			// CR an die Ziffern anhängen
	{
		//if (s[i] == 0)
		//{
			//s[i]= 13;
			//s[i+1] = 10;
			//break;
		//}
	}
	do
	{
		USART0_sendChar(s[i]);	// wait until TX can send
	} while (s[++i] != 0);
}

// Debugging: zwei Int-Variablen anzeigen  
void debug_int2(uint16_t wert, uint16_t wert2)
{
	uint8_t i= 0;
	char s[10] = "";
	char s2[6] = "     ";
	// char * itoa(int val,char * s,int radix)
	itoa(wert,s,10);
	itoa(wert2,s2,10);

	for(uint8_t i=0;i<9;i++)			// Nullen durch Leerzeichen ersetzen
	{
		if (s[i] == 0)
		{
			s[i]=' ';
		}
	}
	
	for(uint8_t i=0;i<9;i++)			// Wert2 einkopieren und CR anhängen
	{
		if (s[i] == ' ')
		{
			for(uint8_t j=0;j<5;j++)
			{
				s[i+1+j]=s2[j];
				if (s[i+1+j] == 0)  s[i+1+j] = 32;
			}
			s[7] = 13; s[8] = 10;
			break;
		}
	}
	
	do
	{
		USART0_sendChar(s[i]);	// wait until TX can send

	} while (s[++i] != 0);
}

// sendet String
void debugSendString(char *str)
{
	for(size_t i = 0; i < strlen(str); i++)
	{
		USART0_sendChar(str[i]);
	}
}

// wie debugSend: char array 
void debug_s(char *s)
{
	volatile int i = 0;
	do
	{
		USART0_sendChar(s[i]);		// wait until TX can send
	} while (s[++i] != 0);
}

// ===========================================================================================
