/*
 * AtTiny3216_I2C_Test.cpp
 *
 * Created: 21.11.2019 23:40:20
 * Author : Micha
 *
 * I2C-Test mit ATTiny3216: Schreiben und Lesen eines EEPROMS 24FC512
 * Ausgabe von Meldungen per USART.
 *
 * Test mit RTC Microchip MCP79410, rudimentre Routinen zum setzen und lesen der Zeit und Datum
 *	
 *	Hardware:
 *		24FC512 EEPROM			ATTiny3216
 *		------------------------------------
 *		A0, A1, A2,GND			GND
 *		Vcc						3,3 Volt
 *		WP						GND
 *		SDA						SDA
 *		SCL						SCL
 *		GND						GND
 *	
 *
 *		MCP79410				ATTiny3216
 *		------------------------------------
 *		A0, A1, A2				VCC 3.3 Volt
 *		WP						GND
 *		SCL						SCL
 *		SDA						SDA	
 *		VCC						3.3 Volt
 *		GND						GND
 */ 


#define F_CPU 3333333UL
#define byte uint8_t

#include <avr/io.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "USART0.h"						// Seriell
#include "I2C_TWI.h"
#include "MCP79410.h"					// RTC




// ======================================================================================================
// EEPROM (24FC512)
// ======================================================================================================

#define i2c_addr 0xA0								// I2C-Adresse Microchip 24LC256 / 24FC256 eeprom
uint16_t eeprom_address = 0;						// aktuelle Adresse im EEPROM

// Ein Byte in externes EEPROM schreiben (24FC512)
void writeEEPROM(uint8_t deviceaddress, uint16_t eeaddress, uint8_t data )
{
	uint8_t result;
	uint8_t msb = (uint8_t) (eeaddress >> 8);
	uint8_t lsb = (uint8_t) (eeaddress & 0xFF);
	
	result = TWI_Start(deviceaddress);
	//if(result == 1) USART0_sendString((char*)"TWI-Start error\r\n");
	
	result = TWI_Write(msb);
	//if(result == 1) USART0_sendString((char*)"TWI-Write error\r\n");

	result = TWI_Write(lsb);
	//if(result == 1) USART0_sendString((char*)"TWI-Write error\r\n");
	
	result = TWI_Write(data);
	//if(result == 1) USART0_sendString((char*)"TWI-Write error\r\n");

	TWI_Stop();

	_delay_ms(50);                                // Zeit fr das EEPROM Daten zu schreiben
}



// Byte aus externem EEPROM lesen (24FC512)
uint8_t readEEPROM(uint8_t deviceaddress, uint16_t eeaddress)
{
	uint8_t result;
	uint8_t rdata = 255;
	uint8_t msb = (uint8_t) (eeaddress >> 8);
	uint8_t lsb = (uint8_t) (eeaddress & 0xFF);
	//Serial.print(msb); Serial.print(" - "); Serial.println(lsb);
	
	result = TWI_Start(deviceaddress);
	//if(result == 1) USART0_sendString((char*)"read: TWI-Start error\r\n");
	
	result = TWI_Write(msb);
	//if(result == 1) USART0_sendString((char*)"read: TWI-Write error\r\n");
	
	result = TWI_Write(lsb);
	//if(result == 1) USART0_sendString((char*)"read: TWI-Write error\r\n");
	
	result = TWI_Start(deviceaddress+1);				// Start mit Read-Bit (Bit0) gesetzt in deviceaddress
	//if(result == 1) USART0_sendString((char*)"read: TWI-Start error\r\n");

	// bei mehreren zu lesenden Bytes wird jeweils mit TWI_Read(1) gelesen, bedeutet Lesen mit ACK
	// nur beim letzten Byte schreibt man TWI_Read(0) mit NACK, dann gibt der Slave den Bus frei fr TWI_Stop();	
	rdata = TWI_Read(NACK);					// Read mit NAK (0), weil letztes zu lesendes Byte
	//if(rdata == 1) USART0_sendString((char*)"read: TWI-Read error\r\n");
		
	TWI_Stop();
	return rdata;
}

int main(void)
{
    /* Replace with your application code */
	uint8_t rd = 0;
	uint8_t data_to_write = 0;
	USART0_init();
	TWI_init();
	USART0_sendString((char *) "Start-Prog\r\n");
	
	jahr = 2019;
	monat = 12;
	tag = 8;
	stunde = 14;
	minute = 58;
	
	char s [20];
	
	// Zeit und Datum einstellen
	MCP79410_DateTime_write(jahr, monat, tag, stunde, minute);
	
    while (1) 
    {
		//// Schreiben in EEPROM, Daten zurcklesen und anzeigen ber USART
		//writeEEPROM(i2c_addr, eeprom_address, data_to_write++ );
		//USART0_sendString((char*)"EEPROM geschrieben\r\n");
		//_delay_ms(1000);
		//
		//rd = readEEPROM(i2c_addr, eeprom_address++);
		//debug_int(rd);
		//USART0_sendString((char *) " - gelesen\r\n");
		
		uint8_t min = MCP79410_Get_Minutes();		// Minuten auslesen
		sprintf(s," Min = %d ", min);
		USART0_sendString((char *)s);
		
		uint8_t sec = MCP79410_Get_Seconds();		// Sekunden auslesen
		sprintf(s," Sec = %d \r\n", sec);
		USART0_sendString((char *)s);
		
		uint16_t year = MCP79410_Get_Year();		// Jahr auslesen
		sprintf(s," Year = %d \r\n", year);
		USART0_sendString((char *)s);
		
		_delay_ms(15000);
    }
}

