/*
 * AtTiny3216_PinChangeInterrupt_Test.cpp
 *
 * Created: 11.11.2019 19:58:38
 * Author : Micha
 *
 * Aufgabe:
 * Demo des Pin-Change-Interrupts eines Ports, legt Prozessor zwischen den Interrupts schlafen 
 * Sleep-Mode: Power-down bei geringem Stromverbrauch
 * weitere Senkung des Stromverbrauchs durch Ausschalten des INPUT BUFFERS Bei jedem nicht benutzten Port-Pin
 *
 * Hardware:
 * Led an PORTC.1
 * Taster gegen Masse an PORTC.0
 * Interrupt durch Rechtecksignal gegen Low an PB4 (1000ms high, min. 5 s LOW)
 *
 * luft!
 */ 

//#define F_CPU 3333333UL								// 3,3 MHz
#define F_CPU 20000000UL								// 20 MHz


#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <avr/sleep.h>
#include "MyPort.h"

/*
To tell the compiler that this variable could be changed outside the scope of its code path analysis 
(e. g. from within an interrupt routine), the variable needs to be declared like:
volatile uint8_t flag; "volatile" ist also wichtig an dieser Stelle!!!		*/
volatile bool myinterrupt = false;				// zeigt empfangenen Interrupt an

void CLKCTRL_init(void)
{
	#if (F_CPU == 1000000UL)                     /* 1MHz, 16/16, set fuse for 16 MHz */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_16X_gc | CLKCTRL_PEN_bm);
	#endif
	
	#if (F_CPU == 20000000UL)                  /* 20MHz, no division */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);
	#endif
	
	#if (F_CPU == 3333333UL)					/* 3.3 MHz bei Fuse = 20 MHz`*/
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */
	#endif

}


// =======================================================================================

void io_init()
{
	// Strom sparen durch das Ausschalten des INPUT_BUFFERS fr nicht genutzte Pins
		PORTA_pin_set_isc(PIN0_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN1_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN2_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN3_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN4_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN5_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN6_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTA_pin_set_isc(PIN7_bm, PORT_ISC_INPUT_DISABLE_gc);
		
		PORTB_pin_set_isc(PIN0_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTB_pin_set_isc(PIN1_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTB_pin_set_isc(PIN2_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTB_pin_set_isc(PIN3_bm, PORT_ISC_INPUT_DISABLE_gc);
		//PORTB_pin_set_isc(PIN4_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTB_pin_set_isc(PIN5_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTB_pin_set_isc(PIN6_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTB_pin_set_isc(PIN7_bm, PORT_ISC_INPUT_DISABLE_gc);

		// PortC.0 und C.1 werden benutzt...weiter gehts mit Pin2
		//PORTC_pin_set_isc(PIN0_bm, PORT_ISC_INPUT_DISABLE_gc);
		//PORTC_pin_set_isc(PIN1_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTC_pin_set_isc(PIN2_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTC_pin_set_isc(PIN3_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTC_pin_set_isc(PIN4_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTC_pin_set_isc(PIN5_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTC_pin_set_isc(PIN6_bm, PORT_ISC_INPUT_DISABLE_gc);
		PORTC_pin_set_isc(PIN7_bm, PORT_ISC_INPUT_DISABLE_gc);

		
		// PortB4 als Pin-Change_Int einrichten
		PORTB.DIRCLR = PIN4_bm;										// ist Eingang
		PORTB.PIN4CTRL |= PORT_PULLUPEN_bm | PORT_ISC_BOTHEDGES_gc;	// Pullup und Int

		// LED		
		PORTC.DIRSET = PIN1_bm;						// PORTC.1 = OUTPUT (LED)

		// **** PORTB.0 sei der Interrupt-Pin ****
		//PORTB.DIR &= ~PIN0_bm;						// PORTB.0 ist Input
		//PORTB.PIN0CTRL |= PORT_PULLUPEN_bm | PORT_ISC_FALLING_gc; 

		// Interrupt-Pin fr Pin-Change ist PC0:
		PORTC.DIRCLR = PIN0_bm;						// Pin PC0 = input (PinChangeInterrupt)
		
		// Pull-Up-Widerstand einschalten, Interrupt bei LOW LEVEL, hier wacht die
		// CPU aus dem Tiefschlaf wieder auf.
		//PORTC.PIN0CTRL |= PORT_PULLUPEN_bm | PORT_ISC_LEVEL_gc;
		
		// Pull-Up-Widerstand einschalten, Interrupt auf beide Flanken, hier wacht die
		// CPU aus dem Tiefschlaf wieder auf.
		PORTC.PIN0CTRL |= PORT_PULLUPEN_bm | PORT_ISC_BOTHEDGES_gc;

		// Das funktioniert nicht:
		// Pull-Up-Widerstand einschalten, Interrupt auf FALLING, dabei wacht der Prozessor 
		// **nicht** wieder aus dem Tiefschlaf (POWER DOWN MODE) auf !!!
		//PORTC.PIN0CTRL |= PORT_PULLUPEN_bm | PORT_ISC_FALLING_gc;

		// Pull-Up-Widerstand einschalten, Interrupt auf RISING, dabei wacht der Prozessor
		// **nicht** wieder aus dem Tiefschlaf (POWER DOWN MODE) auf !!!
		//PORTC.PIN0CTRL |= PORT_PULLUPEN_bm | PORT_ISC_RISING_gc;

		//CPUINT.LVL0PRI = 0 << CPUINT_LVL0PRI_gp;		// Interrupt Level Priority: 0
		//CPUINT.LVL1VEC = 0x0A << CPUINT_LVL1VEC_gp;	// Interrupt Vector with High Priority: 0x08
		
}

int main(void)
{
	CLKCTRL_init();
	io_init();
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sei();
	
    /* Replace with your application code */
    while (1) 
    {
		sleep_mode();
		
		/*
		Sometimes, there are blocks of instructions that need to execute without interruption. 
		They are called atomic blocks.
		This is achieved by disabling interrupts at the beginning of the block and re-enabling them at 
		the end of the block. For this, the ATOMIC_BLOCK macro in <avr/atomic.h> is used.
		*/
		//ATOMIC_BLOCK(ATOMIC_FORCEON)		
		//{
			//_delay_ms(50);						// wird durch Interrupts nicht unterbrochen
												//// wirkt als Entprellung z. B. eines Tasters!
		//}
		
		if(myinterrupt)
		{
			//...
			PORTC.OUTTGL = PIN1_bm;				// Toggle PortC, Pin1 (LED)
			myinterrupt = false;
		}
		
		//_delay_ms(5000);
	}
return 0;
}

// Im Fall, dass PORTB.4 der Interrupt-Pin ist:
ISR(PORTB_PORT_vect)
{
	// here Code...
	if(PORTB.INTFLAGS & PIN4_bm)
	{
		myinterrupt = true;
	}
	PORTB.INTFLAGS |= PORT_INT4_bm;		// Clear Interrupt Pin Flag by writing 1, must do this to handle next interrupt
}

// Fr den Fall, dass PORTC.0 der Interrupt-Pin ist:
ISR(PORTC_PORT_vect)
{
	// here Code...
	//myinterrupt = true;

	// Dieser Vector gilt fr das ganze Port, um einen Interrupt an einem bestimmten Pin anzufragen,
	// schreibt man
	if(PORTC.INTFLAGS & PIN0_bm)
	{
			myinterrupt = true;
	}
	PORTC.INTFLAGS |= PORT_INT0_bm;		// Clear Interrupt Pin Flag by writing 1, must do this to handle next interrupt

}


